@@ -23,6 +23,7 @@ from Products.CMFCore.permissions import View, ModifyPortalContent, SetOwnPasswo
 from Products.CMFCore.utils import getToolByName
 from Products.CMFPlone.browser.navtree import buildFolderTree
 from Products.CMFPlone.CatalogTool import getObjPositionInParent
+from Products.CMFPlone.interfaces import IPloneSiteRoot
 from Products.membrane.interfaces import IUserAuthProvider, IPropertiesProvider, IGroupsProvider, IGroupAwareRolesProvider, IUserChanger
 from Products.Relations.field import RelationField
 from Products.validation import validation
@@ -31,6 +32,7 @@ from ZPublisher.HTTPRequest import HTTPRequest
 from Products.FacultyStaffDirectory.config import *
 from Products.FacultyStaffDirectory.interfaces.person import IPerson
 from Products.FacultyStaffDirectory.interfaces.person import IPersonModifiedEvent
+from Products.FacultyStaffDirectory.interfaces.facultystaffdirectory import IFacultyStaffDirectory
 from Products.FacultyStaffDirectory.permissions import ASSIGN_CLASSIFICATIONS_TO_PEOPLE, ASSIGN_DEPARTMENTS_TO_PEOPLE, ASSIGN_COMMITTIES_TO_PEOPLE, ASSIGN_SPECIALTIES_TO_PEOPLE, CHANGE_PERSON_IDS
 from Products.FacultyStaffDirectory.validators import SequenceValidator
 
@@ -273,10 +275,11 @@ schema = ATContentTypeSchema.copy() + Schema((
             label=u'Departments',
             label_msgid='FacultyStaffDirectory_label_departments',
             i18n_domain='FacultyStaffDirectory',
-            base_query={'portal_type': 'FSDDepartment', 'sort_on': 'sortable_title'},
+            base_query="_search_departments_in_this_fsd",
             allow_browse=0,
             allow_search=1,
             show_results_without_query=1,
+            startup_directory_method="_get_parent_fsd_path",
         ),
         write_permission=ASSIGN_DEPARTMENTS_TO_PEOPLE,
         schemata="Basic Information",
@@ -292,10 +295,11 @@ schema = ATContentTypeSchema.copy() + Schema((
             label=u'Committees',
             label_msgid='FacultyStaffDirectory_label_committees',
             i18n_domain='FacultyStaffDirectory',
-            base_query={'portal_type': 'FSDCommittee', 'sort_on': 'sortable_title'},
+            base_query="_search_committees_in_this_fsd",
             allow_browse=0,
             allow_search=1,
             show_results_without_query=1,
+            startup_directory_method="_get_parent_fsd_path",
         ),
         write_permission=ASSIGN_COMMITTIES_TO_PEOPLE,
         schemata="Professional Information",
@@ -306,11 +310,15 @@ schema = ATContentTypeSchema.copy() + Schema((
     
     RelationField(
         name='specialties',
-        widget=ReferenceBrowserWidget
-        (
+        widget=ReferenceBrowserWidget(
             label=u'Specialties',
             label_msgid='FacultyStaffDirectory_label_specialties',
             i18n_domain='FacultyStaffDirectory',
+            base_query="_search_specialties_in_this_fsd",
+            allow_browse=0,
+            allow_search=1,
+            show_results_without_query=1,
+            startup_directory_method="_get_parent_fsd_path"
         ),
         write_permission=ASSIGN_SPECIALTIES_TO_PEOPLE,
         schemata="Professional Information",
@@ -436,10 +444,12 @@ schema = ATContentTypeSchema.copy() + Schema((
             label=u'Personal Assistant(s)',
             label_msgid='FacultyStaffDirectory_label_assistants',
             i18n_domain='FacultyStaffDirectory',
-            base_query={'portal_type': 'FSDPerson', 'sort_on': 'sortable_title'},
             allow_browse=0,
             allow_search=1,
             show_results_without_query=1,
+            startup_directory_method="_get_parent_fsd_path",
+            base_query="_search_people_in_this_fsd",
+            restrict_browsing_to_startup_directory=True,
         ),
         write_permission="Modify portal content",
         schemata="Basic Information",
@@ -493,7 +503,7 @@ class Person(OrderedBaseFolder, ATCTContent):
         """Notify that the Person has been modified.
         """
         notify(PersonModifiedEvent(self))
-    
+
     security.declareProtected(View, 'at_post_edit_script')
     def at_post_edit_script(self):
         """Notify that the Person has been modified.
@@ -726,16 +736,16 @@ class Person(OrderedBaseFolder, ATCTContent):
             functions, this is what we get.  Can't do it on __init__ since it
             doesn't recognize any of the portal tools for some reason.
         """
-        # Set the startup directory for the specialties field to the SpecialtiesFolder or, failing
-        # that, the root of the FacultyStaffDirectory:
-        urlTool = getToolByName(self, 'portal_url')
-        fsdTool = getToolByName(self, 'facultystaffdirectory_tool')
-        fsd = self.getDirectoryRoot()
-        if fsd and fsd.getSpecialtiesFolder():
-            url = urlTool.getRelativeContentURL(fsd.getSpecialtiesFolder())
-        else:
-            url = ""
-        self.schema['specialties'].widget.startup_directory = '/%s' % url
+        # # Set the startup directory for the specialties field to the SpecialtiesFolder or, failing
+        # # that, the root of the FacultyStaffDirectory:
+        # urlTool = getToolByName(self, 'portal_url')
+        # fsdTool = getToolByName(self, 'facultystaffdirectory_tool')
+        # fsd = self.getDirectoryRoot()
+        # if fsd and fsd.getSpecialtiesFolder():
+        #     url = urlTool.getRelativeContentURL(fsd.getSpecialtiesFolder())
+        # else:
+        #     url = ""
+        # self.schema['specialties'].widget.startup_directory = '/%s' % url
         
         fsd_tool = getToolByName(self,TOOLNAME)
         if (fsd_tool.getPhoneNumberRegex()):
@@ -844,6 +854,75 @@ class Person(OrderedBaseFolder, ATCTContent):
             if password or confirm:
                 if password != confirm:
                     errors['password'] = errors['confirmPassword'] = u'Passwords do not match'
+                    
+    ###
+    # Methods to limit the referenceBrowserWidget start directory and search results    
+    # security.declareProtected(ModifyPortalContent, '_get_parent_fsd_path')
+    # def _get_parent_fsd_path(self):
+    #     """ wrap the utility method so we can use it in the context of an AT Schema declaration
+    #     """
+    #     parent = aq_parent(aq_inner(self))
+    #     if IFacultyStaffDirectory.providedBy(parent):
+    #         # we should pretty much always expect this to be true.  People can't be added anywhere but 
+    #         #   inside an FSD, right at the FSD root, right?  Is this a safe assumption?
+    #         return parent.absolute_url_path()
+    #     else:
+    #         return '/'
+    
+    security.declareProtected(ModifyPortalContent, '_get_parent_fsd_path')
+    def _get_parent_fsd_path(self, relative=True):
+        """ given an object of an FSD type, return the path to the parent FSD of that object
+        """
+        url_tool = getToolByName(self, 'portal_url')
+        # Walk up the tree until you find an FSD
+        parent = aq_parent(aq_inner(self))
+        while not IPloneSiteRoot.providedBy(parent):
+            if IFacultyStaffDirectory.providedBy(parent):
+                if relative:
+                    # get the path relative to the portal root
+                    path = '/'.join(url_tool.getRelativeContentPath(parent))
+                else:
+                    # return the path starting with the portal root
+                    path = parent.absolute_url_path()
+                return path
+            else:
+                parent = aq_parent(aq_inner(parent))
+
+        return ""            
+            
+    security.declareProtected(ModifyPortalContent, '_limit_rbw_search_params')
+    def _limit_rbw_search_params(self, portal_type="FSDPerson", sort_on="sortable_title"):
+        """ return a query dictionary to limit the search parameters for a reference browser
+            widget query.  Use as basis for more specific versions below
+        """
+        path = self._get_parent_fsd_path(relative=False)
+        return {'portal_type': portal_type,
+                'sort_on': sort_on,
+                'path': {'query': path}}
+
+    security.declareProtected(ModifyPortalContent, '_search_people_in_this_fsd')
+    def _search_people_in_this_fsd(self):
+        """ search only parent FSD for only people
+        """
+        return self._limit_rbw_search_params(portal_type="FSDPerson")
+
+    security.declareProtected(ModifyPortalContent, '_search_departments_in_this_fsd')
+    def _search_departments_in_this_fsd(self):
+        """ search only parent FSD for only departments
+        """
+        return self._limit_rbw_search_params(portal_type="FSDDepartment")
+
+    security.declareProtected(ModifyPortalContent, '_search_committees_in_this_fsd')
+    def _search_committees_in_this_fsd(self):
+        """ search only parent FSD for only committees
+        """
+        return self._limit_rbw_search_params(portal_type="FSDCommittee")
+
+    security.declareProtected(ModifyPortalContent, '_search_specialties_in_this_fsd')
+    def _search_specialties_in_this_fsd(self):
+        """ search only parent FSD for only specialties
+        """
+        return self._limit_rbw_search_params(portal_type="FSDSpecialty")
 
 
 # Implementing IMultiPageSchema forces the edit template to render in the more Plone 2.5-ish manner,
